# Copyright (c) 2023, Al Bowles <@akatch>
# Copyright (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
name: unixy
type: stdout
author: Al Bowles (@akatch)
short_description: Condensed Ansible output
description:
  - Consolidated Ansible output in the style of LINUX/UNIX startup logs.
extends_documentation_fragment:
  - ansible.builtin.default_callback
requirements:
  - set as stdout in configuration
"""

from os.path import basename

from ansible import constants as C
from ansible import context
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.callback.default import CallbackModule as CallbackModule_default
from ansible.utils.color import colorize, hostcolor


class CallbackModule(CallbackModule_default):
    """
    Design goals:
    - Print consolidated output that looks like a *NIX startup log
    - Defaults should avoid displaying unnecessary information wherever possible

    TODOs:
    - Only display task names if the task runs on at least one host
    - Add option to display all hostnames on a single line in the appropriate result color (failures may have a separate line)
    - Consolidate stats display
    - Don't show play name if no hosts found
    """

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = "stdout"
    CALLBACK_NAME = "community.general.unixy"

    def _run_is_verbose(self, result):
        return (
            self._display.verbosity > 0 or "_ansible_verbose_always" in result._result
        ) and "_ansible_verbose_override" not in result._result

    def _get_task_display_name(self, task):
        self.task_display_name = None
        display_name = task.get_name().strip().split(" : ")

        task_display_name = display_name[-1]
        if task_display_name.startswith("include"):
            return
        else:
            self.task_display_name = task_display_name

    def _preprocess_result(self, result):
        self.delegated_vars = result._result.get("_ansible_delegated_vars", None)
        self._handle_exception(result._result, use_stderr=self.get_option("display_failed_stderr"))
        self._handle_warnings(result._result)

    def _process_result_output(self, result, msg):
        task_host = result._host.get_name()
        task_result = f"{task_host} {msg}"

        if self._run_is_verbose(result):
            task_result = f"{task_host} {msg}: {self._dump_results(result._result, indent=4)}"
            return task_result

        if self.delegated_vars:
            task_delegate_host = self.delegated_vars.get("ansible_host") or result._task.delegate_to or ""
            task_result = f"{task_host} -> {task_delegate_host} {msg}"

        if result._result.get("msg") and result._result.get("msg") != "All items completed":
            task_result += f" | msg: {to_text(result._result.get('msg'))}"

        if result._result.get("stdout"):
            task_result += f" | stdout: {result._result.get('stdout')}"

        if result._result.get("stderr"):
            task_result += f" | stderr: {result._result.get('stderr')}"

        return task_result

    def v2_playbook_on_task_start(self, task, is_conditional):
        self._get_task_display_name(task)
        if self.task_display_name is not None:
            if task.check_mode and self.get_option("check_mode_markers"):
                self._display.display(f"{self.task_display_name} (check mode)...")
            else:
                self._display.display(f"{self.task_display_name}...")

    def v2_playbook_on_handler_task_start(self, task):
        self._get_task_display_name(task)
        if self.task_display_name is not None:
            if task.check_mode and self.get_option("check_mode_markers"):
                self._display.display(f"{self.task_display_name} (via handler in check mode)... ")
            else:
                self._display.display(f"{self.task_display_name} (via handler)... ")

    def v2_playbook_on_play_start(self, play):
        name = play.get_name().strip()
        if play.check_mode and self.get_option("check_mode_markers"):
            if name and play.hosts:
                msg = f"\n- {name} (in check mode) on hosts: {','.join(play.hosts)} -"
            else:
                msg = "- check mode -"
        else:
            if name and play.hosts:
                msg = f"\n- {name} on hosts: {','.join(play.hosts)} -"
            else:
                msg = "---"

        self._display.display(msg)

    def v2_runner_on_skipped(self, result, ignore_errors=False):
        if self.get_option("display_skipped_hosts"):
            self._preprocess_result(result)
            display_color = C.COLOR_SKIP
            msg = "skipped"

            task_result = self._process_result_output(result, msg)
            self._display.display(f"  {task_result}", display_color)
        else:
            return

    def v2_runner_on_failed(self, result, ignore_errors=False):
        self._preprocess_result(result)
        display_color = C.COLOR_ERROR
        msg = "failed"
        item_value = self._get_item_label(result._result)
        if item_value:
            msg += f" | item: {item_value}"

        task_result = self._process_result_output(result, msg)
        self._display.display(f"  {task_result}", display_color, stderr=self.get_option("display_failed_stderr"))

    def v2_runner_on_ok(self, result, msg="ok", display_color=C.COLOR_OK):
        self._preprocess_result(result)

        result_was_changed = "changed" in result._result and result._result["changed"]
        if result_was_changed:
            msg = "done"
            item_value = self._get_item_label(result._result)
            if item_value:
                msg += f" | item: {item_value}"
            display_color = C.COLOR_CHANGED
            task_result = self._process_result_output(result, msg)
            self._display.display(f"  {task_result}", display_color)
        elif self.get_option("display_ok_hosts"):
            task_result = self._process_result_output(result, msg)
            self._display.display(f"  {task_result}", display_color)

    def v2_runner_item_on_skipped(self, result):
        self.v2_runner_on_skipped(result)

    def v2_runner_item_on_failed(self, result):
        self.v2_runner_on_failed(result)

    def v2_runner_item_on_ok(self, result):
        self.v2_runner_on_ok(result)

    def v2_runner_on_unreachable(self, result):
        self._preprocess_result(result)

        msg = "unreachable"
        display_color = C.COLOR_UNREACHABLE
        task_result = self._process_result_output(result, msg)

        self._display.display(f"  {task_result}", display_color, stderr=self.get_option("display_failed_stderr"))

    def v2_on_file_diff(self, result):
        if result._task.loop and "results" in result._result:
            for res in result._result["results"]:
                if "diff" in res and res["diff"] and res.get("changed", False):
                    diff = self._get_diff(res["diff"])
                    if diff:
                        self._display.display(diff)
        elif "diff" in result._result and result._result["diff"] and result._result.get("changed", False):
            diff = self._get_diff(result._result["diff"])
            if diff:
                self._display.display(diff)

    def v2_playbook_on_stats(self, stats):
        self._display.display("\n- Play recap -", screen_only=True)

        hosts = sorted(stats.processed.keys())
        for h in hosts:
            # TODO how else can we display these?
            t = stats.summarize(h)

            self._display.display(
                f"  {hostcolor(h, t)} : {colorize('ok', t['ok'], C.COLOR_OK)} {colorize('changed', t['changed'], C.COLOR_CHANGED)} "
                f"{colorize('unreachable', t['unreachable'], C.COLOR_UNREACHABLE)} {colorize('failed', t['failures'], C.COLOR_ERROR)} "
                f"{colorize('rescued', t['rescued'], C.COLOR_OK)} {colorize('ignored', t['ignored'], C.COLOR_WARN)}",
                screen_only=True,
            )

            self._display.display(
                f"  {hostcolor(h, t, False)} : {colorize('ok', t['ok'], None)} {colorize('changed', t['changed'], None)} "
                f"{colorize('unreachable', t['unreachable'], None)} {colorize('failed', t['failures'], None)} {colorize('rescued', t['rescued'], None)} "
                f"{colorize('ignored', t['ignored'], None)}",
                log_only=True,
            )
        if stats.custom and self.get_option("show_custom_stats"):
            self._display.banner("CUSTOM STATS: ")
            # per host
            # TODO: come up with 'pretty format'
            for k in sorted(stats.custom.keys()):
                if k == "_run":
                    continue
                stat_val = self._dump_results(stats.custom[k], indent=1).replace("\n", "")
                self._display.display(f"\t{k}: {stat_val}")

            # print per run custom stats
            if "_run" in stats.custom:
                self._display.display("", screen_only=True)
                stat_val_run = self._dump_results(stats.custom["_run"], indent=1).replace("\n", "")
                self._display.display(f"\tRUN: {stat_val_run}")
            self._display.display("", screen_only=True)

    def v2_playbook_on_no_hosts_matched(self):
        self._display.display("  No hosts found!", color=C.COLOR_DEBUG)

    def v2_playbook_on_no_hosts_remaining(self):
        self._display.display("  Ran out of hosts!", color=C.COLOR_ERROR)

    def v2_playbook_on_start(self, playbook):
        if context.CLIARGS["check"] and self.get_option("check_mode_markers"):
            self._display.display(f"Executing playbook {basename(playbook._file_name)} in check mode")
        else:
            self._display.display(f"Executing playbook {basename(playbook._file_name)}")

        # show CLI arguments
        if self._display.verbosity > 3:
            if context.CLIARGS.get("args"):
                self._display.display(
                    f"Positional arguments: {' '.join(context.CLIARGS['args'])}",
                    color=C.COLOR_VERBOSE,
                    screen_only=True,
                )

            for argument in (a for a in context.CLIARGS if a != "args"):
                val = context.CLIARGS[argument]
                if val:
                    self._display.vvvv(f"{argument}: {val}")

    def v2_runner_retry(self, result):
        msg = f"  Retrying... ({result._result['attempts']} of {result._result['retries']})"
        if self._run_is_verbose(result):
            msg += f"Result was: {self._dump_results(result._result)}"
        self._display.display(msg, color=C.COLOR_DEBUG)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_vlan_profiles
short_description: Resource module for networks _vlan _profiles
description:
  - Manage operations create, update and delete of the resource networks _vlan _profiles.
  - Create a VLAN profile for a network.
  - Delete a VLAN profile of a network.
  - Update an existing VLAN profile of a network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  iname:
    description: IName of the profile.
    type: str
  name:
    description: Name of the profile, string length must be from 1 to 255 characters.
    type: str
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  vlanGroups:
    description: An array of VLAN groups.
    elements: dict
    suboptions:
      name:
        description: Name of the VLAN, string length must be from 1 to 32 characters.
        type: str
      vlanIds:
        description: Comma-separated VLAN IDs or ID ranges.
        type: str
    type: list
  vlanNames:
    description: An array of named VLANs.
    elements: dict
    suboptions:
      adaptivePolicyGroup:
        description: Adaptive Policy Group assigned to Vlan ID.
        suboptions:
          id:
            description: Adaptive Policy Group ID.
            type: str
        type: dict
      name:
        description: Name of the VLAN, string length must be from 1 to 32 characters.
        type: str
      vlanId:
        description: VLAN ID.
        type: str
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for networks createNetworkVlanProfile
    description: Complete reference of the createNetworkVlanProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-vlan-profile
  - name: Cisco Meraki documentation for networks deleteNetworkVlanProfile
    description: Complete reference of the deleteNetworkVlanProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-vlan-profile
  - name: Cisco Meraki documentation for networks updateNetworkVlanProfile
    description: Complete reference of the updateNetworkVlanProfile API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-vlan-profile
notes:
  - SDK Method used are
    networks.Networks.create_network_vlan_profile,
    networks.Networks.delete_network_vlan_profile,
    networks.Networks.update_network_vlan_profile,
  - Paths used are
    post /networks/{networkId}/vlanProfiles,
    delete /networks/{networkId}/vlanProfiles/{iname},
    put /networks/{networkId}/vlanProfiles/{iname},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_vlan_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    iname: Profile1
    name: My VLAN profile name
    networkId: string
    vlanGroups:
      - name: named-group-1
        vlanIds: 2,5-7
    vlanNames:
      - adaptivePolicyGroup:
          id: '791'
        name: named-1
        vlanId: '1'
- name: Update by name
  cisco.meraki.networks_vlan_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    iname: string
    name: My VLAN profile name
    networkId: string
    vlanGroups:
      - name: named-group-1
        vlanIds: 2,5-7
    vlanNames:
      - adaptivePolicyGroup:
          id: '791'
        name: named-1
        vlanId: '1'
- name: Delete by name
  cisco.meraki.networks_vlan_profiles:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    iname: string
    networkId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "iname": "string",
      "name": "string",
      "isDefault": true,
      "vlanNames": [
        {
          "name": "string",
          "vlanId": "string",
          "adaptivePolicyGroup": {
            "id": "string",
            "name": "string"
          }
        }
      ],
      "vlanGroups": [
        {
          "name": "string",
          "vlanIds": "string"
        }
      ]
    }
"""



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The iosxr_prefix_lists config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""


from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
)

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.facts.facts import Facts
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.rm_templates.prefix_lists import (
    Prefix_listsTemplate,
)


class Prefix_lists(ResourceModule):
    """
    The iosxr_prefix_lists config class
    """

    def __init__(self, module):
        super(Prefix_lists, self).__init__(
            empty_fact_val=[],
            facts_module=Facts(module),
            module=module,
            resource="prefix_lists",
            tmplt=Prefix_listsTemplate(),
        )
        self.parsers = ["prefix", "description", "prefix_list"]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """
        wantd = {entry["afi"]: entry for entry in self.want}
        haved = {entry["afi"]: entry for entry in self.have}

        self._prefix_list_to_dict(wantd)
        self._prefix_list_to_dict(haved)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in haved.items() if k in wantd or not wantd}
            wantd = {}

        # remove superfluous config for overridden and deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in haved.items():
                if k not in wantd:
                    self._compare(want={}, have=have)

        for k, want in wantd.items():
            self._compare(want=want, have=haved.pop(k, {}))
            # alligning cmd with negate cmd 1st followed by config cmd
        if self.state in ["overridden", "replaced"]:
            self.commands = [each for each in self.commands if "no" in each] + [
                each for each in self.commands if "no" not in each
            ]

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Prefix_lists network resource.
        """
        self._compare_plists(
            want.get("prefix_lists", {}),
            have.get("prefix_lists", {}),
        )

    def _compare_plists(self, want, have):
        for wk, wentry in want.items():
            hentry = have.pop(wk, {})
            # compare sequences
            self._compare_seqs(
                wentry.pop("entries", {}),
                hentry.pop("entries", {}),
            )

        # remove remaining prefix lists
        for h in have.values():
            self.commands.append(
                "no {0} prefix-list {1}".format(h["afi"], h["name"]),
            )

    def _compare_seqs(self, want, have):
        for wseq, wentry in want.items():
            hentry = have.pop(wseq, {})
            self.compare(parsers=self.parsers, want=wentry, have=hentry)

        # remove remaining entries from have prefix list
        for hseq in have.values():
            self.compare(parsers=self.parsers, want={}, have=hseq)

    def _prefix_list_to_dict(self, entry):
        for afi, value in entry.items():
            if "prefix_lists" in value:
                for plist in value["prefix_lists"]:
                    plist.update({"afi": afi})
                    if "entries" in plist:
                        for seq in plist["entries"]:
                            seq.update({"afi": afi, "name": plist["name"]})
                        plist["entries"] = {x["sequence"]: x for x in plist["entries"]}
                value["prefix_lists"] = {
                    (entry["name"], afi): entry for entry in value["prefix_lists"]
                }



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2021, Tim Cragg (@timcragg)
# Copyright: (c) 2023, Akini Ross (@akinross) <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_l3out_interface
short_description: Manage Layer 3 Outside (L3Out) interfaces (l3ext:RsPathL3OutAtt)
description:
- Manage L3Out interfaces on Cisco ACI fabrics.
options:
  tenant:
    description:
    - Name of an existing tenant.
    type: str
    aliases: [ tenant_name ]
    required: true
  l3out:
    description:
    - Name of an existing L3Out.
    type: str
    aliases: [ l3out_name ]
    required: true
  node_profile:
    description:
    - Name of the node profile.
    type: str
    aliases: [ node_profile_name, logical_node ]
    required: true
  interface_profile:
    description:
    - Name of the interface profile.
    type: str
    aliases: [ interface_profile_name, logical_interface ]
    required: true
  pod_id:
    description:
    - Pod to build the interface on.
    type: str
  node_id:
    description:
    - Node to build the interface on for Port-channels and single ports.
    - Hyphen separated pair of nodes (e.g. "201-202") for vPCs.
    type: str
  path_ep:
    description:
    - Path to interface.
    - Interface Policy Group name for Port-channels and vPCs.
    - Port number for single ports (e.g. "eth1/12").
    type: str
  encap:
    description:
    - The encapsulation on the interface (e.g. "vlan-500").
    type: str
  encap_scope:
    description:
    - The scope of the encapsulation on the interface.
    type: str
    choices: [ vrf, local ]
  address:
    description:
    - IP address.
    type: str
    aliases: [ addr, ip_address]
  mtu:
    description:
    - Interface MTU.
    type: str
  ipv6_dad:
    description:
    - IPv6 DAD feature.
    type: str
    choices: [ enabled, disabled]
  interface_type:
    description:
    - Type of interface to build.
    type: str
    choices: [ l3-port, sub-interface, ext-svi ]
  mode:
    description:
    - Interface mode, only used if instance_type is ext-svi.
    type: str
    choices: [ regular, native, untagged ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  auto_state:
    description:
    - SVI auto state.
    type: str
    choices: [ enabled, disabled ]
  description:
    description:
    - The description of the interface.
    type: str
    aliases: [ descr]
  mac:
    description:
    - The MAC address of the interface.
    type: str
    aliases: [ mac_address ]
  micro_bfd:
    description:
    - Enable micro BFD on the interface.
    - Micro BFD should only be configured on Infra SR-MPLS L3Outs Direct Port Channel Interfaces.
    type: bool
  micro_bfd_destination:
    description:
    - The micro BFD destination address of the interface.
    type: str
    aliases: [ micro_bfd_address, micro_bfd_destination_address ]
  micro_bfd_timer:
    description:
    - The micro BFD start timer in seconds.
    - The APIC defaults to C(0) when unset during creation.
    type: int
    aliases: [ micro_bfd_start_timer, micro_bfd_start ]
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The C(tenant), C(l3out), C(node_profile) and the C(interface_profile) must exist before using this module in your playbook.
  The M(cisco.aci.aci_tenant), M(cisco.aci.aci_l3out), M(cisco.aci.aci_l3out_logical_node_profile) and
  M(cisco.aci.aci_l3out_logical_interface_profile) can be used for this.
seealso:
- module: cisco.aci.aci_tenant
- module: cisco.aci.aci_l3out
- module: cisco.aci.aci_l3out_logical_node_profile
- module: cisco.aci.aci_l3out_logical_interface_profile
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(l3ext:RsPathL3OutAtt)
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Tim Cragg (@timcragg)
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Add a new routed interface
  cisco.aci.aci_l3out_interface:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    path_ep: eth1/12
    interface_type: l3-port
    address: 192.168.10.1/27
    state: present
  delegate_to: localhost

- name: Add a new SVI vPC
  cisco.aci.aci_l3out_interface:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201-202
    path_ep: my_vpc_ipg
    interface_type: ext-svi
    encap: vlan-800
    mode: regular
    state: present
  delegate_to: localhost

- name: Add direct port channel interface in the infra SR-MPLS l3out interface profile with micro BFD enabled
  aci_l3out_interface:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: infra
    l3out: ansible_infra_sr_mpls_l3out
    interface_profile: ansible_infra_sr_mpls_l3out_interface_profile
    pod_id: 1
    node_id: 101
    path_ep: pc_ansible_test
    interface_type: l3-port
    addr: 192.168.90.1/24
    micro_bfd: true
    micro_bfd_destination: 192.168.90.2
    micro_bfd_timer: 75
    state: present
  delegate_to: localhost

- name: Delete an interface
  cisco.aci.aci_l3out_interface:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    path_ep: eth1/12
    state: absent
  delegate_to: localhost

- name: Query an interface
  cisco.aci.aci_l3out_interface:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: my_tenant
    l3out: my_l3out
    node_profile: my_node_profile
    interface_profile: my_interface_profile
    pod_id: 1
    node_id: 201
    path_ep: eth1/12
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"], required=True),
        l3out=dict(type="str", aliases=["l3out_name"], required=True),
        node_profile=dict(type="str", aliases=["node_profile_name", "logical_node"], required=True),
        interface_profile=dict(type="str", aliases=["interface_profile_name", "logical_interface"], required=True),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        pod_id=dict(type="str"),
        node_id=dict(type="str"),
        path_ep=dict(type="str"),
        address=dict(type="str", aliases=["addr", "ip_address"]),
        mtu=dict(type="str"),
        ipv6_dad=dict(type="str", choices=["enabled", "disabled"]),
        interface_type=dict(type="str", choices=["l3-port", "sub-interface", "ext-svi"]),
        mode=dict(type="str", choices=["regular", "native", "untagged"]),
        encap=dict(type="str"),
        encap_scope=dict(type="str", choices=["vrf", "local"]),
        auto_state=dict(type="str", choices=["enabled", "disabled"]),
        description=dict(type="str", aliases=["descr"]),
        mac=dict(type="str", aliases=["mac_address"]),
        micro_bfd=dict(type="bool"),
        micro_bfd_destination=dict(type="str", aliases=["micro_bfd_address", "micro_bfd_destination_address"]),
        micro_bfd_timer=dict(type="int", aliases=["micro_bfd_start_timer", "micro_bfd_start"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["interface_type", "pod_id", "node_id", "path_ep"]],
            ["state", "absent", ["pod_id", "node_id", "path_ep"]],
            ["micro_bfd", True, ["micro_bfd_destination"]],
        ],
        required_by={"micro_bfd_timer": "micro_bfd", "micro_bfd_destination": "micro_bfd"},
    )

    aci = ACIModule(module)

    tenant = module.params.get("tenant")
    l3out = module.params.get("l3out")
    node_profile = module.params.get("node_profile")
    interface_profile = module.params.get("interface_profile")
    state = module.params.get("state")
    pod_id = module.params.get("pod_id")
    node_id = module.params.get("node_id")
    path_ep = module.params.get("path_ep")
    address = module.params.get("address")
    mtu = module.params.get("mtu")
    ipv6_dad = module.params.get("ipv6_dad")
    interface_type = module.params.get("interface_type")
    mode = module.params.get("mode")
    encap = module.params.get("encap")
    encap_scope = module.params.get("encap_scope")
    auto_state = module.params.get("auto_state")
    description = module.params.get("description")
    mac = module.params.get("mac")
    micro_bfd = aci.boolean(module.params.get("micro_bfd"))
    micro_bfd_destination = module.params.get("micro_bfd_destination")
    micro_bfd_timer = module.params.get("micro_bfd_timer")

    if node_id and "-" in node_id:
        path_type = "protpaths"
    else:
        path_type = "paths"

    path_dn = None
    if pod_id and node_id and path_ep:
        path_dn = "topology/pod-{0}/{1}-{2}/pathep-[{3}]".format(pod_id, path_type, node_id, path_ep)

    child_classes = []
    child_configs = []
    if micro_bfd is not None:
        child_classes.append("bfdMicroBfdP")
        child_configs.append(dict(bfdMicroBfdP=dict(attributes=dict(adminState=micro_bfd, dst=micro_bfd_destination, stTm=micro_bfd_timer))))

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="l3extOut",
            aci_rn="out-{0}".format(l3out),
            module_object=l3out,
            target_filter={"name": l3out},
        ),
        subclass_2=dict(
            aci_class="l3extLNodeP",
            aci_rn="lnodep-{0}".format(node_profile),
            module_object=node_profile,
            target_filter={"name": node_profile},
        ),
        subclass_3=dict(
            aci_class="l3extLIfP",
            aci_rn="lifp-{0}".format(interface_profile),
            module_object=interface_profile,
            target_filter={"name": interface_profile},
        ),
        subclass_4=dict(
            aci_class="l3extRsPathL3OutAtt",
            aci_rn="rspathL3OutAtt-[{0}]".format(path_dn),
            module_object=path_dn,
            target_filter={"tDn": path_dn},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="l3extRsPathL3OutAtt",
            class_config=dict(
                tDn=path_dn,
                addr=address,
                ipv6Dad=ipv6_dad,
                mtu=mtu,
                ifInstT=interface_type,
                mode=mode,
                encap=encap,
                encapScope="ctx" if encap_scope == "vrf" else encap_scope,
                autostate=auto_state,
                descr=description,
                mac=mac,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="l3extRsPathL3OutAtt")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, nkatarmal-crest <nirav.katarmal@crestdatasys.com>
# Copyright: (c) 2020, Cindy Zhao <cizhao@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: aci_cloud_ctx_profile
short_description:  Manage Cloud Context Profile (cloud:CtxProfile)
description:
- Manage the Cloud Context Profile objects on Cisco Cloud ACI.
notes:
- More information about the internal APIC class B(cloud:CtxProfile) from
  L(the APIC Management Information Model reference,https://developer.cisco.com/docs/apic-mim-ref/).
author:
- Nirav (@crestdatasys)
- Cindy Zhao (@cizhao)
options:
  name:
    description:
    - The name of the Cloud Context Profile
    type: str
    aliases: [ cloud_context_profile ]
  description:
    description:
    - Description of the Cloud Context Profile
    type: str
  name_alias:
    description:
    - An alias for the name of the current object. This relates to the nameAlias field in ACI and is used to rename object without changing the DN
    type: str
  tenant:
    description:
    - The name of the Tenant.
    type: str
  primary_cidr:
    description:
    - The subnet with netmask to use as primary CIDR block for the Cloud Context Profile.
    type: str
  vrf:
    description:
    - The name of the VRF.
    type: str
  region:
    description:
    - The name of the cloud region in which to deploy the network construct.
    type: str
  cloud:
    description:
    - The cloud vendor in which the controller runs.
    choices: [ aws, azure ]
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    choices: [ absent, present, query ]
    type: str
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner
"""

EXAMPLES = r"""
- name: Add a new aci cloud ctx profile
  cisco.aci.aci_cloud_ctx_profile:
    host: apic_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_1
    name: cloud_ctx_profile
    vrf: VRF1
    region: us-west-1
    cloud: aws
    primary_cidr: '10.0.10.1/16'
    state: present
  delegate_to: localhost

- name: Remove an aci cloud ctx profile
  cisco.aci.aci_cloud_ctx_profile:
    host: apic_host
    username: admin
    password: SomeSecretPassword
    tenant: tenant_1
    name: cloud_ctx_profile
    state: absent
  delegate_to: localhost

- name: Query a specific aci cloud ctx profile
  cisco.aci.aci_cloud_ctx_profile:
    host: apic_host
    username: admin
    password: SomeSecretPassword
    tenant: anstest
    name: ctx_profile_1
    state: query
  delegate_to: localhost

- name: Query all aci cloud ctx profile
  cisco.aci.aci_cloud_ctx_profile:
    host: apic_host
    username: admin
    password: SomeSecretPassword
    tenant: anstest
    state: query
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        description=dict(
            type="str",
        ),
        name=dict(type="str", aliases=["cloud_context_profile"]),
        name_alias=dict(
            type="str",
        ),
        tenant=dict(
            type="str",
        ),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        primary_cidr=dict(
            type="str",
        ),
        # FIXME: didn't find the flow_log in UI
        # flow_log=dict(type='str'),
        vrf=dict(type="str"),
        region=dict(type="str"),
        cloud=dict(type="str", choices=["aws", "azure"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name", "tenant"]],
            ["state", "present", ["name", "tenant", "vrf", "region", "primary_cidr", "cloud"]],
        ],
    )

    description = module.params.get("description")
    name = module.params.get("name")
    name_alias = module.params.get("name_alias")
    tenant = module.params.get("tenant")
    state = module.params.get("state")
    primary_cidr = module.params.get("primary_cidr")
    child_configs = []

    vrf = module.params.get("vrf")
    region = module.params.get("region")
    cloud = module.params.get("cloud")

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(aci_class="fvTenant", aci_rn="tn-{0}".format(tenant), target_filter='eq(fvTenant.name, "{0}")'.format(tenant), module_object=tenant),
        subclass_1=dict(
            aci_class="cloudCtxProfile", aci_rn="ctxprofile-{0}".format(name), target_filter='eq(cloudCtxProfile.name, "{0}")'.format(name), module_object=name
        ),
        child_classes=["cloudRsToCtx", "cloudRsCtxProfileToRegion", "cloudRouterP", "cloudCidr"],
    )

    aci.get_existing()

    if state == "present":
        child_configs.append(dict(cloudRsToCtx=dict(attributes=dict(tnFvCtxName=vrf))))
        child_configs.append(dict(cloudRsCtxProfileToRegion=dict(attributes=dict(tDn="uni/clouddomp/provp-{0}/region-{1}".format(cloud, region)))))
        child_configs.append(dict(cloudCidr=dict(attributes=dict(addr=primary_cidr, primary="yes"))))
        aci.payload(
            aci_class="cloudCtxProfile",
            class_config=dict(
                descr=description,
                name=name,
                name_alias=name_alias,
                type="regular",
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="cloudCtxProfile")
        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage Check Point Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_network
short_description: Manages network objects on Check Point over Web Services API
description:
  - Manages network objects on Check Point devices including creating, updating and removing objects.
  - All operations are performed over Web Services API.
  - Available from R80 management version.
version_added: "1.0.0"
author: "Or Soffer (@chkp-orso)"
options:
  name:
    description:
      - Object name.
    type: str
    required: True
  subnet:
    description:
      - IPv4 or IPv6 network address. If both addresses are required use subnet4 and subnet6 fields explicitly.
    type: str
  subnet4:
    description:
      - IPv4 network address.
    type: str
  subnet6:
    description:
      - IPv6 network address.
    type: str
  mask_length:
    description:
      - IPv4 or IPv6 network mask length. If both masks are required use mask-length4 and mask-length6 fields explicitly. Instead of IPv4 mask length
        it is possible to specify IPv4 mask itself in subnet-mask field.
    type: int
  mask_length4:
    description:
      - IPv4 network mask length.
    type: int
  mask_length6:
    description:
      - IPv6 network mask length.
    type: int
  subnet_mask:
    description:
      - IPv4 network mask.
    type: str
  nat_settings:
    description:
      - NAT settings.
    type: dict
    suboptions:
      auto_rule:
        description:
          - Whether to add automatic address translation rules.
        type: bool
      ip_address:
        description:
          - IPv4 or IPv6 address. If both addresses are required use ipv4-address and ipv6-address fields explicitly. This parameter is not
            required in case "method" parameter is "hide" and "hide-behind" parameter is "gateway".
        type: str
      ipv4_address:
        description:
          - IPv4 address.
        type: str
      ipv6_address:
        description:
          - IPv6 address.
        type: str
      hide_behind:
        description:
          - Hide behind method. This parameter is not required in case "method" parameter is "static".
        type: str
        choices: ['gateway', 'ip-address']
      install_on:
        description:
          - Which gateway should apply the NAT translation.
        type: str
      method:
        description:
          - NAT translation method.
        type: str
        choices: ['hide', 'static']
  tags:
    description:
      - Collection of tag identifiers.
    type: list
    elements: str
  broadcast:
    description:
      - Allow broadcast address inclusion.
    type: str
    choices: ['disallow', 'allow']
  color:
    description:
      - Color of the object. Should be one of existing colors.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  groups:
    description:
      - Collection of group identifiers.
    type: list
    elements: str
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_objects
"""

EXAMPLES = """
- name: add-network
  cp_mgmt_network:
    name: New Network 1
    state: present
    subnet: 192.0.2.0
    subnet_mask: 255.255.255.0

- name: set-network
  cp_mgmt_network:
    color: green
    mask_length: 16
    name: New Network 1
    new_name: New Network 2
    state: present
    subnet: 192.0.0.0

- name: delete-network
  cp_mgmt_network:
    name: New Network 2
    state: absent
"""

RETURN = """
cp_mgmt_network:
  description: The checkpoint object created or updated.
  returned: always, except when deleting the object.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_objects,
    api_call,
)


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        subnet=dict(type="str"),
        subnet4=dict(type="str"),
        subnet6=dict(type="str"),
        mask_length=dict(type="int"),
        mask_length4=dict(type="int"),
        mask_length6=dict(type="int"),
        subnet_mask=dict(type="str"),
        nat_settings=dict(
            type="dict",
            options=dict(
                auto_rule=dict(type="bool"),
                ip_address=dict(type="str"),
                ipv4_address=dict(type="str"),
                ipv6_address=dict(type="str"),
                hide_behind=dict(
                    type="str", choices=["gateway", "ip-address"]
                ),
                install_on=dict(type="str"),
                method=dict(type="str", choices=["hide", "static"]),
            ),
        ),
        tags=dict(type="list", elements="str"),
        broadcast=dict(type="str", choices=["disallow", "allow"]),
        color=dict(
            type="str",
            choices=[
                "aquamarine",
                "black",
                "blue",
                "crete blue",
                "burlywood",
                "cyan",
                "dark green",
                "khaki",
                "orchid",
                "dark orange",
                "dark sea green",
                "pink",
                "turquoise",
                "dark blue",
                "firebrick",
                "brown",
                "forest green",
                "gold",
                "dark gold",
                "gray",
                "dark gray",
                "light green",
                "lemon chiffon",
                "coral",
                "sea green",
                "sky blue",
                "magenta",
                "purple",
                "slate blue",
                "violet red",
                "navy blue",
                "olive",
                "orange",
                "red",
                "sienna",
                "yellow",
            ],
        ),
        comments=dict(type="str"),
        details_level=dict(type="str", choices=["uid", "standard", "full"]),
        groups=dict(type="list", elements="str"),
        ignore_warnings=dict(type="bool"),
        ignore_errors=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_objects)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )
    api_call_object = "network"

    result = api_call(module, api_call_object)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# (c) 2020-2024, Alexei Znamensky <russoz@gmail.com>
# Copyright (c) 2020-2024, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.module_utils.common.dict_transformations import dict_merge

from ansible_collections.community.general.plugins.module_utils._mh.base import ModuleHelperBase
from ansible_collections.community.general.plugins.module_utils._mh.mixins.deprecate_attrs import DeprecateAttrsMixin
from ansible_collections.community.general.plugins.module_utils._mh.mixins.state import StateMixin
from ansible_collections.community.general.plugins.module_utils._vardict import VarDict

if t.TYPE_CHECKING:
    from collections.abc import Sequence

    from ansible.module_utils.basic import AnsibleModule


class ModuleHelper(DeprecateAttrsMixin, ModuleHelperBase):
    facts_name: str | None = None
    output_params: Sequence[str] = ()
    diff_params: Sequence[str] = ()
    change_params: Sequence[str] = ()
    facts_params: Sequence[str] = ()

    def __init__(self, module: AnsibleModule | dict[str, t.Any] | None = None) -> None:
        super().__init__(module)

        self.vars = VarDict()
        for name, value in self.module.params.items():  # type: ignore[union-attr]
            self.vars.set(
                name,
                value,
                diff=name in self.diff_params,
                output=name in self.output_params,
                change=None if not self.change_params else name in self.change_params,
                fact=name in self.facts_params,
            )

    def update_vars(self, meta: dict[str, t.Any] | None = None, **kwargs: t.Any) -> None:
        if meta is None:
            meta = {}
        for k, v in kwargs.items():
            self.vars.set(k, v, **meta)

    def update_output(self, **kwargs: t.Any) -> None:
        self.update_vars(meta={"output": True}, **kwargs)

    def update_facts(self, **kwargs: t.Any) -> None:
        self.update_vars(meta={"fact": True}, **kwargs)

    def _vars_changed(self) -> bool:
        return self.vars.has_changed

    def has_changed(self) -> bool:
        return self.changed or self._vars_changed()

    @property
    def output(self) -> dict[str, t.Any]:
        result = dict(self.vars.output())
        if self.facts_name:
            facts = self.vars.facts()
            if facts is not None:
                result["ansible_facts"] = {self.facts_name: facts}
        if self.diff_mode:
            diff = result.get("diff", {})
            vars_diff = self.vars.diff() or {}
            result["diff"] = dict_merge(dict(diff), vars_diff)

        return result


class StateModuleHelper(StateMixin, ModuleHelper):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_devices_statuses_info
short_description: Information module for organizations _devices _statuses
description:
  - Get all organizations _devices _statuses.
  - List the status of every Meraki device in the organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - NetworkIds query parameter. Optional parameter to filter devices by network
        ids.
    elements: str
    type: list
  serials:
    description:
      - Serials query parameter. Optional parameter to filter devices by serials.
    elements: str
    type: list
  statuses:
    description:
      - >
        Statuses query parameter. Optional parameter to filter devices by statuses.
        Valid statuses are "online", "alerting", "offline", "dormant".
    elements: str
    type: list
  productTypes:
    description:
      - >
        ProductTypes query parameter. An optional parameter to filter device statuses
        by product type. Valid types are wireless, appliance, switch, systemsManager,
        camera, cellularGateway, sensor, wirelessController, campusGateway, and secureConnect.
    elements: str
    type: list
  models:
    description:
      - Models query parameter. Optional parameter to filter devices by models.
    elements: str
    type: list
  tags:
    description:
      - >
        Tags query parameter. An optional parameter to filter devices by tags. The
        filtering is case-sensitive. If tags are included, 'tagsFilterType' should
        also be included (see below).
    elements: str
    type: list
  tagsFilterType:
    description:
      - >
        TagsFilterType query parameter. An optional parameter of value 'withAnyTags'
        or 'withAllTags' to indicate whether to return devices which contain ANY or
        ALL of the included tags. If no type is included, 'withAnyTags' will be selected.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationDevicesStatuses
    description: Complete reference of the getOrganizationDevicesStatuses API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-devices-statuses
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_devices_statuses,
  - Paths used are
    get /organizations/{organizationId}/devices/statuses,
"""

EXAMPLES = r"""
- name: Get all organizations _devices _statuses
  cisco.meraki.organizations_devices_statuses_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    serials: []
    statuses: []
    productTypes: []
    models: []
    tags: []
    tagsFilterType: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "name": "string",
        "serial": "string",
        "mac": "string",
        "publicIp": "string",
        "networkId": "string",
        "status": "string",
        "lastReportedAt": "string",
        "lanIp": "string",
        "gateway": "string",
        "ipType": "string",
        "primaryDns": "string",
        "secondaryDns": "string",
        "productType": "string",
        "components": {
          "powerSupplies": [
            {
              "slot": 0,
              "serial": "string",
              "model": "string",
              "status": "string",
              "poe": {
                "unit": "string",
                "maximum": 0
              }
            }
          ]
        },
        "model": "string",
        "tags": [
          "string"
        ]
      }
    ]
"""



# (c) 2020, Alexei Znamensky <russoz@gmail.com>
# Copyright (c) 2020, Ansible Project
# Simplified BSD License (see LICENSES/BSD-2-Clause.txt or https://opensource.org/licenses/BSD-2-Clause)
# SPDX-License-Identifier: BSD-2-Clause

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._mh.deco import module_fails_on_exception
from ansible_collections.community.general.plugins.module_utils._mh.exceptions import ModuleHelperException as _MHE


class ModuleHelperBase:
    module: dict[str, t.Any] | None = None  # TODO: better spec using t.TypedDict
    ModuleHelperException = _MHE
    _delegated_to_module: tuple[str, ...] = (
        "check_mode",
        "get_bin_path",
        "warn",
        "deprecate",
        "debug",
    )

    def __init__(self, module=None):
        self._changed = False

        if module:
            self.module = module

        if not isinstance(self.module, AnsibleModule):
            self.module = AnsibleModule(**self.module)

    @property
    def diff_mode(self):
        return self.module._diff

    @property
    def verbosity(self):
        return self.module._verbosity

    def do_raise(self, *args, **kwargs) -> t.NoReturn:
        raise _MHE(*args, **kwargs)

    def __getattr__(self, attr):
        if attr in self._delegated_to_module:
            return getattr(self.module, attr)
        raise AttributeError(f"ModuleHelperBase has no attribute '{attr}'")

    def __init_module__(self):
        pass

    def __run__(self):
        raise NotImplementedError()

    def __quit_module__(self):
        pass

    def __changed__(self):
        raise NotImplementedError()

    @property
    def changed(self) -> bool:
        try:
            return self.__changed__()
        except NotImplementedError:
            return self._changed

    @changed.setter
    def changed(self, value: bool) -> None:
        self._changed = value

    def has_changed(self):
        raise NotImplementedError()

    @property
    def output(self):
        raise NotImplementedError()

    @module_fails_on_exception
    def run(self):
        self.__init_module__()
        self.__run__()
        self.__quit_module__()
        output = self.output
        if "failed" not in output:
            output["failed"] = False
        self.module.exit_json(changed=self.has_changed(), **output)

    @classmethod
    def execute(cls, module=None):
        cls(module).run()



#!/usr/bin/python

# Copyright (c) 2024, Zoran Krleza (zoran.krleza@true-north.hr)
# Based on code:
# Copyright (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# Copyright (c) 2018, Marcus Watkins <marwatk@marcuswatkins.net>
# Copyright (c) 2013, Phillip Gentry <phillip@cx.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_group_access_token
short_description: Manages GitLab group access tokens
version_added: 8.4.0
description:
  - Creates and revokes group access tokens.
author:
  - Zoran Krleza (@pixslx)
requirements:
  - python-gitlab >= 3.1.0
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes
notes:
  - Access tokens can not be changed. If a parameter needs to be changed, an acceess token has to be recreated. Whether tokens
    are recreated or not is controlled by the O(recreate) option, which defaults to V(never).
  - Token string is contained in the result only when access token is created or recreated. It can not be fetched afterwards.
  - Token matching is done by comparing O(name) option.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  group:
    description:
      - ID or full path of group in the form of group/subgroup.
    required: true
    type: str
  name:
    description:
      - Access token's name.
    required: true
    type: str
  scopes:
    description:
      - Scope of the access token.
      - The values V(read_virtual_registry), V(write_virtual_registry), V(manage_runner), and V(self_rotate) were added in community.general 11.3.0.
    required: true
    type: list
    elements: str
    aliases: ["scope"]
    choices:
      - api
      - read_api
      - read_registry
      - write_registry
      - read_virtual_registry
      - write_virtual_registry
      - read_repository
      - write_repository
      - create_runner
      - manage_runner
      - ai_features
      - k8s_proxy
      - self_rotate
  access_level:
    description:
      - Access level of the access token.
      - The value V(planner) was added in community.general 11.3.0.
    type: str
    default: maintainer
    choices: ["guest", "planner", "reporter", "developer", "maintainer", "owner"]
  expires_at:
    description:
      - Expiration date of the access token in C(YYYY-MM-DD) format.
      - Make sure to quote this value in YAML to ensure it is kept as a string and not interpreted as a YAML date.
    type: str
    required: true
  recreate:
    description:
      - Whether the access token is recreated if it already exists.
      - When V(never) the token is never recreated.
      - When V(always) the token is always recreated.
      - When V(state_change) the token is recreated if there is a difference between desired state and actual state.
    type: str
    choices: ["never", "always", "state_change"]
    default: never
  state:
    description:
      - When V(present) the access token is added to the group if it does not exist.
      - When V(absent) it is removed from the group if it exists.
    default: present
    type: str
    choices: ["present", "absent"]
"""

EXAMPLES = r"""
- name: "Creating a group access token"
  community.general.gitlab_group_access_token:
    api_url: https://gitlab.example.com/
    api_token: "somegitlabapitoken"
    group: "my_group/my_subgroup"
    name: "group_token"
    expires_at: "2024-12-31"
    access_level: developer
    scopes:
      - api
      - read_api
      - read_repository
      - write_repository
    state: present

- name: "Revoking a group access token"
  community.general.gitlab_group_access_token:
    api_url: https://gitlab.example.com/
    api_token: "somegitlabapitoken"
    group: "my_group/my_group"
    name: "group_token"
    expires_at: "2024-12-31"
    scopes:
      - api
      - read_api
      - read_repository
      - write_repository
    state: absent

- name: "Change (recreate) existing token if its actual state is different than desired state"
  community.general.gitlab_group_access_token:
    api_url: https://gitlab.example.com/
    api_token: "somegitlabapitoken"
    group: "my_group/my_group"
    name: "group_token"
    expires_at: "2024-12-31"
    scopes:
      - api
      - read_api
      - read_repository
      - write_repository
    recreate: state_change
    state: present
"""

RETURN = r"""
access_token:
  description:
    - API object.
    - Only contains the value of the token if the token was created or recreated.
  returned: success and O(state=present)
  type: dict
"""

from datetime import datetime

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    find_group,
    gitlab,
    gitlab_authentication,
)

ACCESS_LEVELS = dict(guest=10, planner=15, reporter=20, developer=30, maintainer=40, owner=50)


class GitLabGroupAccessToken:
    def __init__(self, module, gitlab_instance):
        self._module = module
        self._gitlab = gitlab_instance
        self.access_token_object = None

    """
    @param project Project Object
    @param group Group Object
    @param arguments Attributes of the access_token
    """

    def create_access_token(self, group, arguments):
        changed = False
        if self._module.check_mode:
            return True

        try:
            self.access_token_object = group.access_tokens.create(arguments)
            changed = True
        except gitlab.exceptions.GitlabCreateError as e:
            self._module.fail_json(msg=f"Failed to create access token: {e} ")

        return changed

    """
    @param project Project object
    @param group Group Object
    @param name of the access token
    """

    def find_access_token(self, group, name):
        access_tokens = [x for x in group.access_tokens.list(all=True) if not getattr(x, "revoked", False)]
        for access_token in access_tokens:
            if access_token.name == name:
                self.access_token_object = access_token
                return False
        return False

    def revoke_access_token(self):
        if self._module.check_mode:
            return True

        changed = False
        try:
            self.access_token_object.delete()
            changed = True
        except gitlab.exceptions.GitlabCreateError as e:
            self._module.fail_json(msg=f"Failed to revoke access token: {e} ")

        return changed

    def access_tokens_equal(self):
        if self.access_token_object.name != self._module.params["name"]:
            return False
        if self.access_token_object.scopes != self._module.params["scopes"]:
            return False
        if self.access_token_object.access_level != ACCESS_LEVELS[self._module.params["access_level"]]:
            return False
        return self.access_token_object.expires_at == self._module.params["expires_at"]


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        dict(
            state=dict(type="str", default="present", choices=["absent", "present"]),
            group=dict(type="str", required=True),
            name=dict(type="str", required=True),
            scopes=dict(
                type="list",
                required=True,
                aliases=["scope"],
                elements="str",
                choices=[
                    "api",
                    "read_api",
                    "read_registry",
                    "write_registry",
                    "read_virtual_registry",
                    "write_virtual_registry",
                    "read_repository",
                    "write_repository",
                    "create_runner",
                    "manage_runner",
                    "ai_features",
                    "k8s_proxy",
                    "self_rotate",
                ],
            ),
            access_level=dict(
                type="str",
                default="maintainer",
                choices=["guest", "planner", "reporter", "developer", "maintainer", "owner"],
            ),
            expires_at=dict(type="str", required=True),
            recreate=dict(type="str", default="never", choices=["never", "always", "state_change"]),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
        ],
        required_together=[["api_username", "api_password"]],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"]],
        supports_check_mode=True,
    )

    state = module.params["state"]
    group_identifier = module.params["group"]
    name = module.params["name"]
    scopes = module.params["scopes"]
    access_level_str = module.params["access_level"]
    expires_at = module.params["expires_at"]
    recreate = module.params["recreate"]

    access_level = ACCESS_LEVELS[access_level_str]

    try:
        datetime.strptime(expires_at, "%Y-%m-%d")
    except ValueError:
        module.fail_json(msg="Argument expires_at is not in required format YYYY-MM-DD")

    gitlab_instance = gitlab_authentication(module)

    gitlab_access_token = GitLabGroupAccessToken(module, gitlab_instance)

    group = find_group(gitlab_instance, group_identifier)
    if group is None:
        module.fail_json(msg=f"Failed to create access token: group {group_identifier} does not exists")

    gitlab_access_token_exists = False
    gitlab_access_token.find_access_token(group, name)
    if gitlab_access_token.access_token_object is not None:
        gitlab_access_token_exists = True

    if state == "absent":
        if gitlab_access_token_exists:
            gitlab_access_token.revoke_access_token()
            module.exit_json(changed=True, msg=f"Successfully deleted access token {name}")
        else:
            module.exit_json(changed=False, msg="Access token does not exists")

    if state == "present":
        if gitlab_access_token_exists:
            if gitlab_access_token.access_tokens_equal():
                if recreate == "always":
                    gitlab_access_token.revoke_access_token()
                    gitlab_access_token.create_access_token(
                        group, {"name": name, "scopes": scopes, "access_level": access_level, "expires_at": expires_at}
                    )
                    module.exit_json(
                        changed=True,
                        msg="Successfully recreated access token",
                        access_token=gitlab_access_token.access_token_object._attrs,
                    )
                else:
                    module.exit_json(
                        changed=False,
                        msg="Access token already exists",
                        access_token=gitlab_access_token.access_token_object._attrs,
                    )
            else:
                if recreate == "never":
                    module.fail_json(
                        msg="Access token already exists and its state is different. It can not be updated without recreating."
                    )
                else:
                    gitlab_access_token.revoke_access_token()
                    gitlab_access_token.create_access_token(
                        group, {"name": name, "scopes": scopes, "access_level": access_level, "expires_at": expires_at}
                    )
                    module.exit_json(
                        changed=True,
                        msg="Successfully recreated access token",
                        access_token=gitlab_access_token.access_token_object._attrs,
                    )
        else:
            gitlab_access_token.create_access_token(
                group, {"name": name, "scopes": scopes, "access_level": access_level, "expires_at": expires_at}
            )
            if module.check_mode:
                module.exit_json(changed=True, msg="Successfully created access token", access_token={})
            else:
                module.exit_json(
                    changed=True,
                    msg="Successfully created access token",
                    access_token=gitlab_access_token.access_token_object._attrs,
                )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_wireless_controller_devices_interfaces_l2_statuses_change_history_by_device_info
short_description: Information module for organizations _wireless _controller _devices
  _interfaces l2 _statuses _change _history _by _device
description:
  - Get all organizations _wireless _controller _devices _interfaces l2 _statuses
    _change _history _by _device.
  - List wireless LAN controller layer 2 interfaces history status in an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  serials:
    description:
      - >
        Serials query parameter. Optional parameter to filter wireless LAN controller
        by its cloud ID. This filter uses multiple exact matches.
    elements: str
    type: list
  includeInterfacesWithoutChanges:
    description:
      - >
        IncludeInterfacesWithoutChanges query parameter. By default, interfaces without
        changes are omitted from the response for brevity. If you want to include
        the interfaces even if they have no changes, set to true. (default false).
    type: bool
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 31 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 31 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 31 days. The default is 7 days.
    type: float
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wirelessController getOrganizationWirelessControllerDevicesInterfacesL2StatusesChangeHistoryByDevice
    description: Complete reference of the getOrganizationWirelessControllerDevicesInterfacesL2StatusesChangeHistoryByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-wireless-controller-devices-interfaces-l2-statuses-change-history-by-device
notes:
  - SDK Method used are
    wireless_controller.WirelessController.get_organization_wireless_controller_devices_interfaces_l2_statuses_change_history_by_device,
  - Paths used are
    get /organizations/{organizationId}/wirelessController/devices/interfaces/l2/statuses/changeHistory/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _wireless _controller _devices _interfaces l2 _statuses
    _change _history _by _device
  cisco.meraki.organizations_wireless_controller_devices_interfaces_l2_statuses_change_history_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    serials: []
    includeInterfacesWithoutChanges: true
    t0: string
    t1: string
    timespan: 0
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "serial": "string",
          "interfaces": [
            {
              "name": "string",
              "mac": "string",
              "changes": [
                {
                  "ts": "string",
                  "status": "string",
                  "warnings": [
                    "string"
                  ],
                  "errors": [
                    "string"
                  ]
                }
              ]
            }
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):

    # Standard files documentation fragment
    DOCUMENTATION = r'''
options:
    total_pages:
        description:
          - total_pages(int), use with perPage to get total results up to total_pages*perPage; -1 for all pages
        type: str
        required: true
    direction:
        description:
          - direction (string), direction to paginate, either "next" (default) or "prev" page
        type: str
        default: https://api.meraki.com/api/v1
'''



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_compute_info
version_added: "3.15.0"
short_description: List ML Compute resources
description:
    - List ML Compute resources.
options:
    name:
        description:
            - Name of the Azure ML compute.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    type:
        description:
            - The type of compute target.
        choices:
            - amlcompute
            - computeinstance
        required: false
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: List ML Compute
  azure.azcollection.azure_rm_ml_compute_info:
    name: MyCompute
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List ML Compute
  azure.azcollection.azure_rm_ml_compute_info:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    type: amlcompute
'''

RETURN = '''
ml_computes:
    description:
        - Compute that was just created or updated.
    returned: always
    type: dict
    sample: [
      {
        "description": "Compute Integration Tests",
        "enable_node_public_ip": true,
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-compute/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx/computes/amlxxxxxxx-xxxcomp",
        "idle_time_before_scale_down": 120,
        "location": "eastus",
        "max_instances": 200,
        "min_instances": 0,
        "name": "amlxxxxxxx-xxxcomp",
        "network_settings": {},
        "provisioning_state": "Succeeded",
        "size": "STANDARD_DS3_V2",
        "ssh_public_access_enabled": true,
        "tags": {"created_by": "Ansible"},
        "tier": "dedicated",
        "type": "amlcompute"
      },
      {
        "created_on": "2026-02-16T19:08:51.821821+0000",
        "description": "Compute Integration Tests",
        "enable_node_public_ip": true,
        "enable_os_patching": false,
        "enable_root_access": true,
        "enable_sso": true,
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-compute/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxx-xxx/computes/compxxxxxxx-xxxinst",
        "identity": {
          "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
          "type": "system_assigned"
        },
        "last_operation": {
          "operation_name": "Stop",
          "operation_status": "Succeeded",
          "operation_time": "2026-02-16T19:12:01.237Z",
          "operation_trigger": "User"
        },
        "location": "eastus",
        "name": "compxxxxxxx-xxxinst",
        "network_settings": {
          "private_ip_address": "10.0.0.4",
          "public_ip_address": "xx.xxx.xxx.xxx"
        },
        "os_image_metadata": {
          "current_image_version": "26.01.05",
          "is_latest_os_image_version": true,
          "latest_image_version": "26.01.05"
        },
        "provisioning_state": "Succeeded",
        "release_quota_on_stop": false,
        "services": [
          {
            "display_name": "Jupyter",
            "endpoint_uri": "https://compxxxxxxx-xxxinst.eastus.instances.azureml.ms/tree/"
          },
          {
            "display_name": "Jupyter Lab",
            "endpoint_uri": "https://compxxxxxxx-xxxinst.eastus.instances.azureml.ms/lab"
          }
        ],
        "size": "Standard_DS3_v2",
        "ssh_public_access_enabled": false,
        "ssh_settings": {
          "admin_username": "azureuser",
          "ssh_key_value": "ssh-rsa AAAAB........PLp/ administrator@xxxxxx-xxxxxxx",
          "ssh_port": "50000"
        },
        "state": "Stopped",
        "tags": {
          "created_by": "Ansible"
        },
        "type": "computeinstance"
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLComputeInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            type=dict(
                type='str',
                required=False,
                choices=['amlcompute', 'computeinstance'],
            ),
        )

        self._client = None
        self.ml_compute = None
        self.ml_registry = None

        self.results = dict(
            ml_computes=[]
        )

        super(AzureRMMLComputeInfo, self).__init__(self.module_arg_spec,
                                                   supports_tags=False,
                                                   supports_check_mode=True,
                                                   facts_module=True,
                                                   )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            try:
                result = self.client.compute.get(self.name)
                ml_computes = [self.entity_to_dict(result)]
            except ResourceNotFoundError:
                ml_computes = []
        else:
            results = self.client.compute.list(compute_type=self.type)
            ml_computes = [self.entity_to_dict(x) for x in results]

        self.results['ml_computes'] = ml_computes
        return self.results


def main():
    AzureRMMLComputeInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_traffic_shaping_uplink_bandwidth_info
short_description: Information module for networks _appliance _traffic _shaping _uplink
  _bandwidth
description:
  - Get all networks _appliance _traffic _shaping _uplink _bandwidth. - > Returns
    the uplink bandwidth limits for your MX network. This may not reflect the affected
    device's hardware capabilities. For more information on your device's hardware
    capabilities, please consult our MX Family Datasheet - https //meraki.cisco.com/product-collateral/mx-family-datasheet/?file
    .
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance getNetworkApplianceTrafficShapingUplinkBandwidth
    description: Complete reference of the getNetworkApplianceTrafficShapingUplinkBandwidth
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-appliance-traffic-shaping-uplink-bandwidth
notes:
  - SDK Method used are
    appliance.Appliance.get_network_appliance_traffic_shaping_uplink_bandwidth,
  - Paths used are
    get /networks/{networkId}/appliance/trafficShaping/uplinkBandwidth,
"""

EXAMPLES = r"""
- name: Get all networks _appliance _traffic _shaping _uplink _bandwidth
  cisco.meraki.networks_appliance_traffic_shaping_uplink_bandwidth_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "wan1": {
        "limitUp": 0,
        "limitDown": 0
      },
      "wan2": {
        "limitUp": 0,
        "limitDown": 0
      },
      "cellular": {
        "limitUp": 0,
        "limitDown": 0
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: backup_selection_info
version_added: 6.0.0
short_description: Describe AWS Backup Selections
description:
  - Lists info about Backup Selection configuration for a given Backup Plan.
author:
  - Gomathi Selvi Srinivasan (@GomathiselviS)
  - Kristof Imre Szabo (@krisek)
  - Alina Buzachis (@alinabuzachis)
options:
  backup_plan_name:
    description:
      - Uniquely identifies the backup plan to be associated with the selection of resources.
    required: true
    type: str
    aliases:
      - plan_name
  backup_selection_names:
    description:
      - Uniquely identifies the backup plan the selections should be listed for.
    type: list
    elements: str
    aliases:
     - selection_names
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.
- name: Gather information about all backup selections
  amazon.aws.backup_selection_info:
    backup_plan_name: "{{ backup_plan_name }}"

- name: Gather information about a particular backup selection
  amazon.aws.backup_selection_info:
    backup_plan_name: "{{ backup_plan_name }}"
    backup_selection_names:
      - "{{ backup_selection_name }}"
"""

RETURN = r"""
backup_selections:
    description: List of backup selection objects. Each element consists of a dict with all the information related to that backup selection.
    type: list
    elements: dict
    returned: always
    contains:
        backup_plan_id:
            description: Backup plan id.
            returned: always
            type: str
            sample: "1111f877-1ecf-4d79-9718-a861cd09df3b"
        creation_date:
            description: Backup plan creation date.
            returned: always
            type: str
            sample: "2023-01-24T10:08:03.193000+01:00"
        iam_role_arn:
            description: IAM role arn.
            returned: always
            type: str
            sample: "arn:aws:iam::111122223333:role/system-backup"
        selection_id:
            description: Backup selection id.
            returned: always
            type: str
            sample: "1111c217-5d71-4a55-8728-5fc4e63d437b"
        selection_name:
            description: Backup selection name.
            returned: always
            type: str
            sample: elastic
        conditions:
            description: List of conditions (expressed as a dict) that are defined to assign resources to the backup plan using tags.
            returned: always
            type: dict
            sample: {}
        list_of_tags:
            description: Conditions defined to assign resources to the backup plans using tags.
            returned: always
            type: list
            elements: dict
            sample: []
        not_resources:
            description: List of Amazon Resource Names (ARNs) that are excluded from the backup plan.
            returned: always
            type: list
            sample: []
        resources:
            description: List of Amazon Resource Names (ARNs) that are assigned to the backup plan.
            returned: always
            type: list
            sample: []
"""


try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.backup import get_selection_details
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def main():
    argument_spec = dict(
        backup_plan_name=dict(type="str", required=True, aliases=["plan_name"]),
        backup_selection_names=dict(type="list", elements="str", aliases=["selection_names"]),
    )
    result = {}

    module = AnsibleAWSModule(argument_spec=argument_spec, supports_check_mode=True)

    try:
        client = module.client("backup", retry_decorator=AWSRetry.jittered_backoff())
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    result["backup_selections"] = get_selection_details(
        module, client, module.params.get("backup_plan_name"), module.params.get("backup_selection_names")
    )
    module.exit_json(**camel_dict_to_snake_dict(result))


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: cp_mgmt_delete_rules_batch
short_description: Delete rules in batch from the same layer. Use this API to achieve optimum performance when removing more than one rule.
description:
  - Delete rules in batch from the same layer. Use this API to achieve optimum performance when removing more than one rule.
  - Warnings are ignored when using this API, operation will apply changes while ignoring warnings.
  - Supported rules types are access-rule, nat-rule, https-rule and threat-exception.
  - This module is not idempotent.
  - All operations are performed over Web Services API.
  - Available from R81.10 JHF management version.
version_added: "3.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  objects:
    description:
      - Batch of rules separated by types.
    type: list
    elements: dict
    suboptions:
      layer:
        description:
          - Layer name or uid.
        type: str
      type:
        description:
          - Type of rules to be deleted. <br>Only types from above are supported.
        type: str
      list:
        description:
          - List of rules from the same type to be deleted. <br>Use the "delete" API reference documentation for a single rule command to find the
            expected fields for the request.<br>For example, to delete access-rule, use the "delete-access-rule" command found in the API reference
            documentation (under Access Control & NAT). <br>Note, "ignore-errors", "ignore-warnings" and "details-level" options are not supported when
            deleting a batch of objects.
        type: list
        elements: dict
  auto_publish_session:
    description:
    - Publish the current session if changes have been performed after task completes.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: delete-rules-batch
  cp_mgmt_delete_rules_batch:
    objects:
      - layer: Network
        list:
          - rule_number: 1
          - rule_number: 2
        type: access-rule
      - layer: Standard
        list:
          - rule_number: 1
          - rule_number: 2
        type: nat-rule
      - layer: Default Layer
        list:
          - rule_number: 1
          - rule_number: 2
        type: https-rule
"""

RETURN = """
cp_mgmt_delete_rules_batch:
  description: The checkpoint delete-rules-batch output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import (
    checkpoint_argument_spec_for_commands,
    api_command,
)


def main():
    argument_spec = dict(
        objects=dict(
            type="list",
            elements="dict",
            options=dict(
                layer=dict(type="str"),
                type=dict(type="str"),
                list=dict(type="list", elements="dict"),
            ),
        ),
        auto_publish_session=dict(type="bool"),
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "delete-rules-batch"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_policies_global_group_policies_firewall_rulesets_assignments
short_description: Resource module for organizations _policies _global _group _policies
  _firewall _rulesets _assignments
description:
  - Manage operations create, update and delete of the resource organizations _policies
    _global _group _policies _firewall _rulesets _assignments.
  - Create an Organization-Wide Policy Ruleset Assignment.
  - Delete an Organization-Wide Policy Ruleset Assignment.
  - Update an Organization-Wide Policy Ruleset Assignment.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  assignmentId:
    description: AssignmentId path parameter. Assignment ID.
    type: str
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  policyId:
    description: ID of the policy to assign the ruleset to.
    type: str
  priority:
    description: Priority of the ruleset assignment (lower numbers = higher priority).
    type: int
  rulesetId:
    description: ID of the ruleset to assign.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createOrganizationPoliciesGlobalGroupPoliciesFirewallRulesetsAssignment
    description: Complete reference of the createOrganizationPoliciesGlobalGroupPoliciesFirewallRulesetsAssignment
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-organization-policies-global-group-policies-firewall-rulesets-assignment
  - name: Cisco Meraki documentation for organizations deleteOrganizationPoliciesGlobalGroupPoliciesFirewallRulesetsAssignment
    description: Complete reference of the deleteOrganizationPoliciesGlobalGroupPoliciesFirewallRulesetsAssignment
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-organization-policies-global-group-policies-firewall-rulesets-assignment
  - name: Cisco Meraki documentation for organizations updateOrganizationPoliciesGlobalGroupPoliciesFirewallRulesetsAssignment
    description: Complete reference of the updateOrganizationPoliciesGlobalGroupPoliciesFirewallRulesetsAssignment
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-organization-policies-global-group-policies-firewall-rulesets-assignment
notes:
  - SDK Method used are
    organizations.Organizations.create_organization_policies_global_group_policies_firewall_rulesets_assignment,
    organizations.Organizations.delete_organization_policies_global_group_policies_firewall_rulesets_assignment,
    organizations.Organizations.update_organization_policies_global_group_policies_firewall_rulesets_assignment,
  - Paths used are
    post /organizations/{organizationId}/policies/global/group/policies/firewall/rulesets/assignments,
    delete /organizations/{organizationId}/policies/global/group/policies/firewall/rulesets/assignments/{assignmentId},
    put /organizations/{organizationId}/policies/global/group/policies/firewall/rulesets/assignments/{assignmentId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_policies_global_group_policies_firewall_rulesets_assignments:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    organizationId: string
    policyId: '789'
    priority: 0
    rulesetId: '456'
- name: Update by id
  cisco.meraki.organizations_policies_global_group_policies_firewall_rulesets_assignments:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    assignmentId: string
    organizationId: string
    policyId: '789'
    priority: 0
    rulesetId: '456'
- name: Delete by id
  cisco.meraki.organizations_policies_global_group_policies_firewall_rulesets_assignments:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    assignmentId: string
    organizationId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "assignmentId": "string",
      "rulesetId": "string",
      "policyId": "string",
      "priority": 0,
      "createdAt": "string",
      "lastUpdatedAt": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_add_custom_trusted_ca_certificate
short_description: Create new custom trusted CA certificate.
description:
  - Create new custom trusted CA certificate.
  - All operations are performed over Web Services API.
  - Available from R82 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  base64_certificate:
    description:
      - Certificate file encoded in base64.<br/>Valid file formats, x509.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: add-custom-trusted-ca-certificate
  cp_mgmt_add_custom_trusted_ca_certificate:
    base64_certificate:
      "MIIEkzCCAnugAwIBAgIVAO5SRZQELwNNhWF+8st6ox9uXYgeMA0GCSqGSIb3DQEBCwUAMIGrMQswCQYDVQQGEwJJTDEPMA0GA1UECBMGSXNyYWVsMS4wLAYDVQQKEyVDaGVja1BvaW50IFNvZnR3YXJlIF
       lY2hub2xvZ2llcyBMVEQuMQwwCgYDVQQLEwNNSVMxIjAgBgNVBAMTGUNoZWNrUG9pbnQtU1NMLUluc3BlY3Rpb24xKTAnBgkqhkiG9w0BCQEWGmlsX3NlY3VyaXR5QGNoZWNrcG9pbnQuY29tMB4XDTIzM
       MxMzAwMDAwMFoXDTIzMDYxMTIzNTk1OVowbzELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEzARBgNVBAcTCk1lbmxvIFBhcmsxHTAbBgNVBAoTFE1ldGEgUGxhdGZvcm1zLCBJbmMuMRcw
       QYDVQQDDA4qLndoYXRzYXBwLm5ldDBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IABPjo05vRHAJYYWx55SOu2b1ZIQPOOtJNipSBXf1BFBDQhrkp20YTA296MzKii2j3TgVi/1t44cW5mD1RWobfAQujgbM
       gbAwHQYDVR0lBBYwFAYIKwYBBQUHAwEGCCsGAQUFBwMCMHQGA1UdEQRtMGuCDioud2hhdHNhcHAubmV0ghIqLmNkbi53aGF0c2FwcC5uZXSCEiouc25yLndoYXRzYXBwLm5ldIIOKi53aGF0c2FwcC5jb2
       CBXdhLm1lggx3aGF0c2FwcC5jb22CDHdoYXRzYXBwLm5ldDAOBgNVHQ8BAf8EBAMCBaAwCQYDVR0TBAIwADANBgkqhkiG9w0BAQsFAAOCAgEAA/sIadLr9ahEVq8h9HuofHODUuzxVFulAZu8uSiyY4ACb
       Hcvm36MYQCzYV56t4fe+I++ls8KAESZgdE0KoD5/6efzK05Ufok+y15QexAR5AxZlJqtoHIuc7iOolPbkLW77GKrbgfEgmwOCX9/86Pug4ZSrrBUPPt9i3accNkAP+SH9Lft1geS2E/q+xcRhbhDcYTYD5
       X0MiEv0UaAzwS3adWAZbD7R42u+xNCpX8iUyiwp2UvMf0l/+Q8CAtw4D5s/8hD7Vqvrv4H/ZfV7SrZ+rPrihi01t6LlcpZ2YMucX/tSgDzkjYWmT26V2OgRklM0aQWvHD3DVpghIJfI2swAAJJ5wvqwcJe
       WHAQb3aQZgHXjGF/LyBYCQsohTHUL7rhL8CxNlDTNhN2e+NRFGYGer157RCmM8xKroe3/X9pYifbzyEWInqQ+ycmLsQyAd7pPW+W1K1tlk9Niqk3dNQ10daYGau3IPWF5+iHtOlWjLcQrSj60Uv7Ebi0E+
       Oe0tDabunCj6SEauGFxeJhM9xUZnOwb5wqIt+uGqPQ9WRJLehqwdFhiWOqwUfNcksn7l0M6e9Mnkh1J2kGxamQ0bvK7ftpm5O8MTAft0y882IfC++Zuk4gLhQoeE3s6877/rrHRJB/H8ZUaaBxAi2qH0NZ
       ParXUxOkil5rVgFqI="
"""

RETURN = """
cp_mgmt_add_custom_trusted_ca_certificate:
  description: The checkpoint add-custom-trusted-ca-certificate output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        base64_certificate=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full'])
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "add-custom-trusted-ca-certificate"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python

# Copyright (c) 2022, Christian Wollinger <@cwollinger>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: alerta_customer
short_description: Manage customers in Alerta
version_added: 4.8.0
description:
  - Create or delete customers in Alerta with the REST API.
author: Christian Wollinger (@cwollinger)
seealso:
  - name: API documentation
    description: Documentation for Alerta API.
    link: https://docs.alerta.io/api/reference.html#customers
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  customer:
    description:
      - Name of the customer.
    required: true
    type: str
  match:
    description:
      - The matching logged in user for the customer.
    required: true
    type: str
  alerta_url:
    description:
      - The Alerta API endpoint.
    required: true
    type: str
  api_username:
    description:
      - The username for the API using basic auth.
    type: str
  api_password:
    description:
      - The password for the API using basic auth.
    type: str
  api_key:
    description:
      - The access token for the API.
    type: str
  state:
    description:
      - Whether the customer should exist or not.
      - Both O(customer) and O(match) identify a customer that should be added or removed.
    type: str
    choices: [absent, present]
    default: present
"""

EXAMPLES = r"""
- name: Create customer
  community.general.alerta_customer:
    alerta_url: https://alerta.example.com
    api_username: admin@example.com
    api_password: password
    customer: Developer
    match: dev@example.com

- name: Delete customer
  community.general.alerta_customer:
    alerta_url: https://alerta.example.com
    api_username: admin@example.com
    api_password: password
    customer: Developer
    match: dev@example.com
    state: absent
"""

RETURN = r"""
msg:
  description:
    - Success or failure message.
  returned: always
  type: str
  sample: Customer customer1 created
response:
  description:
    - The response from the API.
  returned: always
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import basic_auth_header, fetch_url


class AlertaInterface:
    def __init__(self, module):
        self.module = module
        self.state = module.params["state"]
        self.customer = module.params["customer"]
        self.match = module.params["match"]
        self.alerta_url = module.params["alerta_url"]
        self.headers = {"Content-Type": "application/json"}

        if module.params.get("api_key", None):
            self.headers["Authorization"] = f"Key {module.params['api_key']}"
        else:
            self.headers["Authorization"] = basic_auth_header(
                module.params["api_username"], module.params["api_password"]
            )

    def send_request(self, url, data=None, method="GET"):
        response, info = fetch_url(self.module, url, data=data, headers=self.headers, method=method)

        status_code = info["status"]
        if status_code == 401:
            self.module.fail_json(failed=True, response=info, msg=f"Unauthorized to request '{method}' on '{url}'")
        elif status_code == 403:
            self.module.fail_json(failed=True, response=info, msg=f"Permission Denied for '{method}' on '{url}'")
        elif status_code == 404:
            self.module.fail_json(failed=True, response=info, msg=f"Not found for request '{method}' on '{url}'")
        elif status_code in (200, 201):
            return self.module.from_json(response.read())
        self.module.fail_json(failed=True, response=info, msg=f"Alerta API error with HTTP {status_code} for {url}")

    def get_customers(self):
        url = f"{self.alerta_url}/api/customers"
        response = self.send_request(url)
        pages = response["pages"]
        if pages > 1:
            for page in range(2, pages + 1):
                page_url = f"{url}?page={page}"
                new_results = self.send_request(page_url)
                response.update(new_results)
        return response

    def create_customer(self):
        url = f"{self.alerta_url}/api/customer"

        payload = {
            "customer": self.customer,
            "match": self.match,
        }

        payload = self.module.jsonify(payload)
        response = self.send_request(url, payload, "POST")
        return response

    def delete_customer(self, id):
        url = f"{self.alerta_url}/api/customer/{id}"

        response = self.send_request(url, None, "DELETE")
        return response

    def find_customer_id(self, customer):
        for i in customer["customers"]:
            if self.customer == i["customer"] and self.match == i["match"]:
                return i["id"]
        return None


def main():
    module = AnsibleModule(
        argument_spec=dict(
            state=dict(choices=["present", "absent"], default="present"),
            customer=dict(type="str", required=True),
            match=dict(type="str", required=True),
            alerta_url=dict(type="str", required=True),
            api_username=dict(type="str"),
            api_password=dict(type="str", no_log=True),
            api_key=dict(type="str", no_log=True),
        ),
        required_together=[["api_username", "api_password"]],
        mutually_exclusive=[["api_username", "api_key"]],
        supports_check_mode=True,
    )

    alerta_iface = AlertaInterface(module)

    if alerta_iface.state == "present":
        response = alerta_iface.get_customers()
        if alerta_iface.find_customer_id(response):
            module.exit_json(changed=False, response=response, msg=f"Customer {alerta_iface.customer} already exists")
        else:
            if not module.check_mode:
                response = alerta_iface.create_customer()
            module.exit_json(changed=True, response=response, msg=f"Customer {alerta_iface.customer} created")
    else:
        response = alerta_iface.get_customers()
        id = alerta_iface.find_customer_id(response)
        if id:
            if not module.check_mode:
                alerta_iface.delete_customer(id)
            module.exit_json(
                changed=True, response=response, msg=f"Customer {alerta_iface.customer} with id {id} deleted"
            )
        else:
            module.exit_json(changed=False, response=response, msg=f"Customer {alerta_iface.customer} does not exists")


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2026 Zun Yang (@zunyangc)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_appconfiguration
version_added: "3.15.0"
short_description: Manage Azure App Configuration stores.
description:
    - Create, update and delete Azure App Configuration stores.

options:
    resource_group:
        description:
            - The name of the Resource Group to which the App Configuration store belongs.
        required: True
        type: str
    appconfig_name:
        description:
            - Name of the app configuration store.
        required: True
        type: str
    identity:
        description:
            - Managed identity configuration for the store.
        type: dict
        suboptions:
            type:
                description:
                    - Type of the managed identity
                required: true
                choices:
                    - SystemAssigned
                    - UserAssigned
                    - SystemAssigned, UserAssigned
                type: str
            user_assigned_identity_ids:
                description:
                    - List of user-assigned identity resource IDs (required when type includes UserAssigned).
                required: false
                type: list
                elements: str
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    sku:
        description:
            - SKU tier of the store.
        type: str
        choices:
            - 'free'
            - 'developer'
            - 'standard'
            - 'premium'
    enable_purge_protection:
        description:
            - Property specifying whether protection against purge is enabled for this store.
        type: bool
        default: False
    soft_delete_retention_in_days:
        description:
            - Property specifying the number of days to retain deleted store.
        type: int
    public_network_access:
        description:
            - Public network access setting for the store.
            - If omitted at creation, Azure treats it as C(Automatic). Once set to C(Enabled) or C(Disabled), it cannot be returned to C(Automatic);
              omit the property in updates to leave unchanged.
        type: str
        choices:
            - Disabled
            - Enabled
    disable_local_auth:
        description:
            - Property specifying whether local authentication (access keys) is enabled for the store.
        type: bool
        default: False
    replicas:
        description:
            - List of replicas to manage for this store.
        type: list
        elements: dict
        suboptions:
            name:
                description:
                    - Name of the replica.
                required: true
                type: str
            location:
                description:
                    - Location of the replica.
                required: true
                type: str
    is_purge_deleted:
        description:
            - Whether permanently deletes the specified store. aka Purges the deleted Azure App Configuration store
            - When I(is_purge_deleted) is specified, the I(location) has to be configured.
            - If not configured, the default location of the resource group will be used.
        type: bool
        default: False
    state:
        description:
            - Assert the state of the App Configuration store. Use C(present) to create or update an App Configuration store and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Zun Yang (@zunyangc)

'''

EXAMPLES = '''
- name: Create a Standard App Configuration store with system-assigned identity
  azure.azcollection.azure_rm_appconfiguration:
    resource_group: myResourceGroup
    appconfig_name: myAppConfigStore
    location: eastus
    sku: standard
    identity:
      type: SystemAssigned
    enable_local_auth: true
    tags:
      env: prod
    state: present

- name: Assign a user-assigned identity and enable purge protection
  azure.azcollection.azure_rm_appconfiguration:
    resource_group: myResourceGroup
    appconfig_name: myAppConfigStore
    identity:
      type: UserAssigned
      user_assigned_identity_ids:
        - /subscriptions/xxxx/resourceGroups/myResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/myIdentity
    enable_purge_protection: true
    soft_delete_retention_in_days: 7
    state: present

- name: Delete an App Configuration store
  azure.azcollection.azure_rm_appconfiguration:
    resource_group: myRG
    appconfig_name: my-appconf-legacy
    state: absent

- name: Purge a soft-deleted App Configuration store (subscription-level permission required)
  azure.azcollection.azure_rm_appconfiguration:
    resource_group: myRG
    appconfig_name: my-appconf-legacy
    location: eastus
    is_purge_deleted: true
    state: absent

- name: Create store with two replicas
  azure.azcollection.azure_rm_appconfiguration:
    resource_group: myRG
    appconfig_name: myappconf
    location: eastus
    sku: standard
    replicas:
      - name: r1
        location: westus
      - name: r2
        location: northeurope
    state: present
'''

RETURN = '''
id:
    description:
        - The Azure Resource Manager resource ID for the App Configuration store.
    returned: always
    type: str
    sample: /subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/myRG/providers/Microsoft.AppConfiguration/configurationStores/myAppConfigStore
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt

try:
    from azure.core.polling import LROPoller
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError
    from azure.mgmt.appconfiguration import AppConfigurationManagementClient
except ImportError:
    # Handled in azure_rm_common_ext
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMAppConfiguration(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            appconfig_name=dict(
                type='str',
                required=True
            ),
            identity=dict(
                type='dict',
                options=dict(
                    type=dict(
                        type='str',
                        required=True,
                        choices=['SystemAssigned', 'UserAssigned', 'SystemAssigned, UserAssigned']
                    ),
                    user_assigned_identity_ids=dict(
                        type='list',
                        elements='str'
                    )
                )
            ),
            location=dict(type='str'),
            sku=dict(
                type='str',
                choices=['free', 'developer', 'standard', 'premium']
            ),
            enable_purge_protection=dict(
                type='bool',
                default=False
            ),
            soft_delete_retention_in_days=dict(
                type='int'
            ),
            public_network_access=dict(
                type='str',
                choices=['Disabled', 'Enabled']
            ),
            disable_local_auth=dict(
                type='bool',
                default=False
            ),
            replicas=dict(
                type='list',
                elements='dict',
                options=dict(
                    name=dict(type='str', required=True),
                    location=dict(type='str', required=True)
                )
            ),
            # Administrative op: purge soft-deleted store
            is_purge_deleted=dict(type='bool', default=False),

            # Azure tags (provided by AzureRMModuleBaseExt)
            # 'tags' is injected by base class when supports_tags=True

            state=dict(
                type='str',
                default='present',
                choices=['absent', 'present']
            )
        )

        self.resource_group = None
        self.appconfig_name = None
        self.identity = None
        self.is_purge_deleted = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction
        self.parameters = dict()
        self.tags = None

        super(AzureRMAppConfiguration, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                      supports_tags=True,
                                                      supports_check_mode=True)

    @property
    def managed_identity(self):
        return {}

    def exec_module(self, **kwargs):
        """Main module execution method"""

        # Bind known attributes and build parameters payload
        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if key in kwargs:
                setattr(self, key, kwargs[key])
            if key in kwargs and kwargs.get(key) is not None:
                if key == 'location':
                    self.parameters['location'] = kwargs[key]
                elif key == 'sku':
                    # ARM expects {"sku": {"name": "<tier>"}}
                    self.parameters.setdefault('sku', {})['name'] = kwargs[key]
                elif key == "public_network_access":
                    self.parameters.setdefault("properties", {})["publicNetworkAccess"] = kwargs[key]
                elif key == "disable_local_auth":
                    self.parameters.setdefault("properties", {})["disableLocalAuth"] = kwargs[key]
                elif key == "enable_purge_protection":
                    self.parameters.setdefault("properties", {})["enablePurgeProtection"] = kwargs[key]
                elif key == "soft_delete_retention_in_days":
                    self.parameters.setdefault("properties", {})["softDeleteRetentionInDays"] = kwargs[key]

        self.mgmt_client = self.get_mgmt_svc_client(AppConfigurationManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        # Resolve RG + default location
        resource_group = self.get_resource_group(self.resource_group)
        if "location" not in self.parameters:
            self.parameters['location'] = resource_group.location

        # Validation: SKU vs purge fields
        if self.sku not in ('standard', 'premium'):
            if self.enable_purge_protection:
                self.fail("enable_purge_protection is only supported on 'standard' or 'premium' SKUs.")
            if self.soft_delete_retention_in_days is not None:
                self.fail("soft_delete_retention_in_days is only supported on 'standard' or 'premium' SKUs.")

        # Normalize identity changes using base helper
        identity_dict = None
        if self.identity:
            id_type = (self.identity.get("type") or "").strip()
            ids = self.identity.get("user_assigned_identity_ids") or []
            if "UserAssigned" in id_type and not ids:
                self.fail("identity.user_assigned_identity_ids must be provided when identity.type includes 'UserAssigned'.")
            if id_type == "SystemAssigned":
                identity_dict = {"type": "SystemAssigned"}
            elif id_type == "UserAssigned":
                identity_dict = {"type": "UserAssigned",
                                 "userAssignedIdentities": {uid: {} for uid in ids}}
            elif id_type == "SystemAssigned, UserAssigned":
                identity_dict = {"type": "SystemAssigned, UserAssigned",
                                 "userAssignedIdentities": {uid: {} for uid in ids}}
            else:
                # If not provided or unrecognized, omit identity block altogether
                identity_dict = None

            if identity_dict:
                self.parameters["identity"] = identity_dict

        # Read current
        old_response = self.get_instance()
        response = None

        # Compute identity_changed
        identity_changed = False
        if old_response and identity_dict:
            old_ident = old_response.get('identity') or {}
            old_type = (old_ident.get('type') or '').lower()
            new_type = (identity_dict.get('type') or '').lower()
            if old_type != new_type:
                identity_changed = True
            else:
                # Compare UAMI sets if applicable
                if 'userassigned' in new_type:
                    old_ids = set(((old_ident.get('user_assigned_identities') or {}).keys()) or [])
                    new_ids = set(((identity_dict.get('userAssignedIdentities') or {}).keys()) or [])
                    if old_ids != new_ids:
                        identity_changed = True

        # Decide action
        if not old_response:
            if self.state == 'absent':
                self.log("Old instance didn't exist")
                self.to_do = Actions.NoAction
            else:
                self.to_do = Actions.Create
        else:
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                # compute update deltas
                self.to_do = self._needs_update(old_response, self.parameters, identity_changed)

        # Execute
        if self.to_do in (Actions.Create, Actions.Update):
            self.results['changed'] = True
            if not self.check_mode:
                self.parameters["tags"] = self.tags
                response = self.create_or_update_store()
                if self.replicas is not None:
                    self.reconcile_replicas(self.replicas)
                if response is None:
                    response = self.get_instance()
        elif self.to_do == Actions.Delete:
            self.results['changed'] = True
            if not self.check_mode:
                self.delete_store()
        elif self.to_do == Actions.NoAction and self.replicas is not None:
            self.reconcile_replicas(self.replicas)
            if response is None:
                response = old_response or self.get_instance()
        else:
            # NoAction
            self.results['changed'] = False
            response = old_response

        # Purge soft-deleted if requested
        if self.is_purge_deleted:
            purge_response = self.get_deleted(self.parameters["location"])
            if purge_response:
                self.results['changed'] = True
                if not self.check_mode:
                    self.purge_deleted(self.parameters["location"])

        if response is not None and hasattr(response, "as_dict"):
            response = response.as_dict()

        if response:
            self.results["id"] = response["id"]

        return self.results

    def get_instance(self):
        '''
        Get the properties of the specified App Configuration store.

        :return: deserialized App Configuration store instance state dictionary.
        '''
        try:
            response = self.mgmt_client.configuration_stores.get(resource_group_name=self.resource_group,
                                                                 config_store_name=self.appconfig_name)
            return response.as_dict()
        except ResourceNotFoundError:
            return False

    def create_or_update_store(self):
        '''
        Create or update (PUT) App Configuration store with the specified configuration.

        :return: deserialized App Configuration store state dictionary.
        '''
        try:
            poller = self.mgmt_client.configuration_stores.begin_create(resource_group_name=self.resource_group,
                                                                        config_store_name=self.appconfig_name,
                                                                        config_store_creation_parameters=self.parameters)
            if isinstance(poller, LROPoller):
                return self.get_poller_result(poller)
            return poller.as_dict() if hasattr(poller, "as_dict") else poller
        except Exception as exc:
            self.fail("Error creating/updating App Configuration store '{}': {}".format(self.appconfig_name, str(exc)))

    def delete_store(self):
        '''
        Delete the specified App Configuration store.

        :return: True.
        '''
        try:
            poller = self.mgmt_client.configuration_stores.begin_delete(resource_group_name=self.resource_group,
                                                                        config_store_name=self.appconfig_name)
            if isinstance(poller, LROPoller):
                return self.get_poller_result(poller)
            return True
        except Exception as exc:
            self.fail("Error deleting the App Configuration store '{}': {}".format(self.appconfig_name, str(exc)))

    def get_deleted(self, location):
        '''
        Get deleted store
        :return: True or False
        '''
        try:
            self.mgmt_client.configuration_stores.get_deleted(self.appconfig_name, location)
        except Exception as e:
            self.log('Error attempting to get the deleted store: {0}'.format(str(e)))
            return False
        return True

    def purge_deleted(self, location):
        '''
        Purge the specified soft-deleted App Configuration store.
        '''
        try:
            poller = self.mgmt_client.configuration_stores.begin_purge_deleted(location=location,
                                                                               config_store_name=self.appconfig_name)
            if isinstance(poller, LROPoller):
                return self.get_poller_result(poller)
            return True
        except HttpResponseError as e:
            # 404/NotFound => nothing to purge
            status = getattr(e, 'status_code', None) or getattr(getattr(e, 'response', None), 'status_code', None)
            if status == 404:
                return False
            self.fail("Error purging deleted App Configuration store '{0}' in '{1}': {2}".format(
                self.appconfig_name, location, str(e)))
        except Exception as exc:
            self.fail("Error purging deleted App Configuration store '{0}' in '{1}': {2}".format(
                self.appconfig_name, location, str(exc)))

    def _list_replicas(self):
        '''
        List replicas for the specified App Configuration store.

        :return: Dictionary of replicas.
        '''
        found = {}
        pager = self.mgmt_client.replicas.list_by_configuration_store(
            resource_group_name=self.resource_group,
            config_store_name=self.appconfig_name
        )
        for r in pager:
            rd = r.as_dict()
            found[rd['name']] = {'location': rd.get('location'), 'id': rd.get('id')}
        return found

    def _create_replica(self, name, location):
        '''
        Create a replica for the specified App Configuration store.

        :return: deserialized replica instance state dictionary.
        '''
        poller = self.mgmt_client.replicas.begin_create(
            resource_group_name=self.resource_group,
            config_store_name=self.appconfig_name,
            replica_name=name,
            replica_creation_parameters={'location': location, 'properties': {}}
        )
        return self.get_poller_result(poller)

    def _delete_replica(self, name):
        '''
        Delete a replica for the specified App Configuration store.

        :return: True.
        '''
        poller = self.mgmt_client.replicas.begin_delete(
            resource_group_name=self.resource_group,
            config_store_name=self.appconfig_name,
            replica_name=name
        )
        return self.get_poller_result(poller)

    def reconcile_replicas(self, desired):
        '''
        Reconcile replicas to match desired state.
        Authoritative: create missing, delete extras, recreate on location change.
        '''
        # desired: list[{'name': ..., 'location': ...}]
        # authoritative behavior: create missing, delete extras, (re)create on location change
        desired_index = {x['name']: x['location'] for x in (desired or [])}
        current = self._list_replicas()

        # create or recreate when location changed
        for name, loc in desired_index.items():
            if name not in current:
                self.results['changed'] = True
                if not self.check_mode:
                    self._create_replica(name, loc)
            else:
                if loc and (loc.lower() != (current[name].get('location') or '').lower()):
                    self.results['changed'] = True
                    if not self.check_mode:
                        self._delete_replica(name)
                        self._create_replica(name, loc)

        # delete extras (authoritative)
        extra = set(current.keys()) - set(desired_index.keys())
        for name in sorted(extra):
            self.results['changed'] = True
            if not self.check_mode:
                self._delete_replica(name)

    def _needs_update(self, old, new_params, identity_changed):
        '''
        Determine whether an update is required by comparing fields.
        Explicitly manage. Keep parity with Azure defaults where possible.

        :return: Action Required.
        '''
        to_do = Actions.NoAction
        props_new = new_params.get('properties') or {}

        # sku
        sku_old = ((old.get('sku') or {}).get('name') or '').lower()
        sku_new = ((new_params.get('sku') or {}).get('name') or '').lower()
        if sku_new and sku_new != sku_old:
            # Note: provider/API might restrict downgrades; PUT will handle errors.
            to_do = Actions.Update

        # publicNetworkAccess
        if 'publicNetworkAccess' in props_new:
            if (props_new.get('publicNetworkAccess') or '').lower() != (old.get('public_network_access') or '').lower():
                to_do = Actions.Update

        # disableLocalAuth
        if 'disableLocalAuth' in props_new:
            if bool(props_new.get('disableLocalAuth')) != bool(old.get('disable_local_auth')):
                to_do = Actions.Update

        # enablePurgeProtection
        if 'enablePurgeProtection' in props_new:
            if bool(props_new.get('enablePurgeProtection')) != bool(old.get('enable_purge_protection')):
                to_do = Actions.Update

        # softDeleteRetentionInDays
        if 'softDeleteRetentionInDays' in props_new:
            if props_new.get('softDeleteRetentionInDays') != old.get('soft_delete_retention_in_days'):
                to_do = Actions.Update

        # tags
        update_tags, newtags = self.update_tags(old.get('tags', {}) or {})
        if update_tags:
            self.tags = newtags
            to_do = Actions.Update

        # identity
        if identity_changed:
            to_do = Actions.Update

        # Write back potentially adjusted props
        if props_new:
            new_params['properties'] = props_new

        return to_do


def main():
    """Main execution"""
    AzureRMAppConfiguration()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_inventory_onboarding_cloud_monitoring_networks_info
short_description: Information module for organizations _inventory _onboarding _cloud
  _monitoring _networks
description:
  - Get all organizations _inventory _onboarding _cloud _monitoring _networks.
  - Returns list of networks eligible for adding cloud monitored device.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  deviceType:
    description:
      - DeviceType query parameter. Device Type switch or wireless controller.
    type: str
  search:
    description:
      - Search query parameter. Optional parameter to search on network name.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 100000. Default is 1000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationInventoryOnboardingCloudMonitoringNetworks
    description: Complete reference of the getOrganizationInventoryOnboardingCloudMonitoringNetworks
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-inventory-onboarding-cloud-monitoring-networks
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_inventory_onboarding_cloud_monitoring_networks,
  - Paths used are
    get /organizations/{organizationId}/inventory/onboarding/cloudMonitoring/networks,
"""

EXAMPLES = r"""
- name: Get all organizations _inventory _onboarding _cloud _monitoring _networks
  cisco.meraki.organizations_inventory_onboarding_cloud_monitoring_networks_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    deviceType: string
    search: string
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "organizationId": "string",
        "name": "string",
        "productTypes": [
          "string"
        ],
        "timeZone": "string",
        "tags": [
          "string"
        ],
        "enrollmentString": "string",
        "url": "string",
        "notes": "string",
        "isBoundToConfigTemplate": true
      }
    ]
"""
